@@ -220,7 +220,11 @@ class MessageManager:
         return list(self.db_instances_sta.keys())
 
     def dumps_db_data(self):
-        return self.engine.db_data._getvalue()
+        db_data = self.engine.db_data._getvalue()
+        for k, elem in db_data.items():
+            del elem['args']['log_path']
+        return db_data
+
 
     def dumps_db_type(self):
         return self.engine.db_type._getvalue()
@@ -600,11 +604,63 @@ class MessageManager:
         in_cmd.update({'timestamp_in': timestamp()})
         group = msg.get('group')
         COMMAND = self.commands.get(c_key, {})
-        bprint("=====INTEPRETER=======")
-        rprint("COMMAND>%s" % COMMAND)
-        bprint("MSG IN %s" % in_cmd)
-        bprint("=====INTEPRETER=======")
         if group in COMMAND.get('group'):
             coro_callback = self.msg_process.get(c_key)
-            bprint("Coroutine->%s" % coro_callback)
             return await coro_callback(idm, in_cmd)
+
+
+from dataclasses import dataclass
+from gnss_collector.engine.steps import Logger
+import sys
+from tblib import pickling_support
+pickling_support.install()
+from dataclasses_json import dataclass_json
+
+@dataclass_json
+@dataclass
+class MSGException:
+    exc_type: str = ""
+    value: str = ""
+    traceback: BaseException = None
+
+    def check(self):
+        return self.traceback is None
+    
+    @property
+    def info(self):
+        return (self.exc_type, self.value, self.traceback)
+
+
+    def __str__(self):
+        return f"{self.exc_type} : {self.value}"
+    
+@dataclass_json
+@dataclass
+class MessageLog:
+    func_name: str
+    #timestamp: datetime
+    level: Logger
+    message: str
+    error: MSGException
+
+    @property
+    def msg(self):
+        return f"{self.func_name} :: {self.message}"
+
+    @property
+    def exception(self):
+        if isinstance(self.error, MSGException):
+            return self.error.check()
+        else:
+            return True
+
+    @property
+    def exc(self):
+        return self.error.info
+    
+    @property
+    def log(self):
+        return (self.level, self.msg)
+
+    def __str__(self):
+        return f"{self.func_name} :: {self.message} :: Error :> {self.exception}"
