@@ -1,31 +1,42 @@
 #!/usr/bin/env python
 # stdlib
+import sys
+import re
+import time
+import json
 import asyncio
 import uvloop
-import concurrent.futures
-import sys
-import time
 import functools
+import concurrent.futures
 import multiprocessing as mp
-import re
 from pathlib import Path
-# contrib
-import click
-# gnss libs
 from multiprocessing import Manager, Queue
 from multiprocessing.managers import BaseManager
+from loky import get_reusable_executor
+
+# contrib
+import click
+from rich import print
+
+# gnss libs
 from networktools.environment import get_env_variable
 from networktools.colorprint import gprint, bprint, rprint
 from networktools.ssh import clean_port
-from tasktools.taskloop import coromask, renew, simple_fargs
+from networktools.time import timestamp, now
+from networktools.messages import MSGException, MessageLog
+
+# tasktools loop async
 from tasktools.assignator import TaskAssignator
-from collector import Engine, deactive_server
+
+# deprecate socket service
 from gnsocket.gn_socket import GNCSocket
 from gnsocket.socket_server import GNCSocketServer
+
+# own module
+from gnss_collector import Engine, deactive_server, Logger, rdbnow
+from gnss_collector.scripts import EnvData
+
 # desde la misma carpeta
-from collector.scripts import EnvData
-
-
 # fn que maneja excepción en socket
 def socket_exception(ex, gs, idc):
     bprint("Excepción en socket")
@@ -76,7 +87,8 @@ def get_conf_data(conf):
     return dbdata
 
 def start_collector(env, conf="collector_example.json"):
-    new_loop =uvloop.new_event_loop()
+    exc = MSGException()
+    new_loop = uvloop.new_event_loop()
     asyncio.set_event_loop(new_loop)
     mp.set_start_method('spawn')
     data = {}
@@ -89,10 +101,11 @@ def start_collector(env, conf="collector_example.json"):
         data = get_conf_data(conf)
         clean_port(data.get("COLLECTOR_SOCKET_PORT"))
     workers = data.get("COLLECTOR_WORKERS")
-    address = list(map(lambda k:data.get(k),["COLLECTOR_SOCKET_IP","COLLECTOR_SOCKET_PORT"]))
-    server_name=data.get("SERVER_NAME")
+    address = [data.get(k) for k in ("COLLECTOR_SOCKET_IP","COLLECTOR_SOCKET_PORT")]
+    server_name = data.get("SERVER_NAME")
     print("Workers", workers)
     with concurrent.futures.ProcessPoolExecutor(workers) as executor:
+    #with get_reusable_executor(max_workers=workers) as executor:
         loop = asyncio.get_event_loop()
         manager = Manager()
         # terminal->network
@@ -103,7 +116,6 @@ def start_collector(env, conf="collector_example.json"):
         # Manager dict to share betwen process
         stations = manager.dict()
         wait = manager.dict()
-        enqueued = manager.list()
         dbtype = manager.dict()
         protocol = manager.dict()
         protocols = manager.dict()
@@ -124,7 +136,9 @@ def start_collector(env, conf="collector_example.json"):
         inc_msg = manager.dict()
         status_tasks = manager.dict()
         free_ids = manager.dict()
-        stime = 1
+        changes = manager.dict() # check for changes, change control
+        # value
+        sleep_time = 1
         ids = manager.list()
         idd = manager.list()
         ipt = manager.list()
@@ -137,24 +151,29 @@ def start_collector(env, conf="collector_example.json"):
         queue_n2t = manager.Queue()
         # network->terminal
         queue_t2n = manager.Queue()
+        queue_log = manager.Queue()
+        queue_db = manager.Queue()
+        args = []
         kwargs = {
-            "server":data.get("server_name".upper()),
+            "server": data.get("server_name".upper()),
             "log_path": data.get("log_path".upper()),
-            'dt_criteria':10,
-            "timeout":15,
-            "raise_timeout":False,
-            'dbdata':data.get("dbdata".upper())
+            'dt_criteria': 4,
+            "timeout": 15,
+            "raise_timeout": False,
+            'dbdata': data.get("dbdata".upper())
         }
         queue_list = [queue_n2t,
                       queue_t2n,
                       queue_process,
-                      queue_ans_process, ]
+                      queue_ans_process, 
+                      queue_db,
+                      queue_log]
+        print("Pre engine kwargs", kwargs)
         engine = Engine(
             queue_list,
-            stime,
+            sleep_time,
             data.get("est_by_proc".upper()),
             stations,
-            enqueued,
             dbtype,
             protocol,
             status_sta,
@@ -172,6 +191,7 @@ def start_collector(env, conf="collector_example.json"):
             ipt,
             idm,
             ico,
+            changes,
             data.get("gsof_timeout".upper()),
             sta_init,
             db_init,
@@ -181,12 +201,12 @@ def start_collector(env, conf="collector_example.json"):
             idc,
             data.get("rdb_address".upper()),
             8,
-            **kwargs
+            args,
+            kwargs
         )
-        NIPT = workers-3
-
+        NIPT = workers-2
+        gprint(f"Number of process {NIPT}")
         # activate new assignator
-        sta_asssigned = []
         cll_status = data.get("cll_status".upper())
         cll_group = data.get("cll_group".upper())        
         assignator = TaskAssignator(
@@ -196,8 +216,10 @@ def start_collector(env, conf="collector_example.json"):
             assigned_tasks,
             cll_status,
             cll_group,
-            enqueued,
             locker)
+        message = f"Assignator created"
+        
+        queue_log.put({"run_collector": [MessageLog(rdbnow(), "run_collector", message, Logger.INFO, exc).rdb]})
 
         try:
             print("Obteniendo lista inicial de estaciones")
@@ -215,10 +237,21 @@ def start_collector(env, conf="collector_example.json"):
         print("Está corriendo loop?", loop.is_running())
 
         try:
+            # this includes log tasks
             loop.run_in_executor(
                 executor,
                 engine.msg_network_task
             )
+            message = f"Msg network task running at one process"
+            
+            queue_log.put(
+                {
+                    "run_collector": [
+                        MessageLog(rdbnow(),"run_collector", message,
+                                   Logger.INFO, exc).rdb
+                    ]
+                 }
+            )            
             print("Está corriendo loop on msg networktask?", loop.is_running())            
         except Exception as ex:
             print("Falla encargar networktask")
@@ -229,25 +262,25 @@ def start_collector(env, conf="collector_example.json"):
         print("Socket path", socket_path, type(socket_path))
 
         opts = {
-            "timeout":10,
-            "raise_timeout":False, 
+            "timeout": 10,
+            "raise_timeout": False, 
             'server': server_name,
         }
 
         print("Socket opts", opts)
         opts["log_path"] = socket_path
         print("Iniciando socket server", address)
-        server_socket = GNCSocketServer(
-            queue_n2t, queue_t2n, address=address,
-            callback_exception=socket_exception, **opts)
-        try:
-            loop.run_in_executor(
-                executor,
-                server_socket.socket_task
-            )
-        except Exception as ex:
-            print("Falla en cargar socket task")
-            raise ex
+        # server_socket = GNCSocketServer(
+        #     queue_n2t, queue_t2n, address=address,
+        #     callback_exception=socket_exception, **opts)
+        # try:
+        #     loop.run_in_executor(
+        #         executor,
+        #         server_socket.socket_task
+        #     )
+        # except Exception as ex:
+        #     print("Falla en cargar socket task")
+        #     raise ex
 
         ######
 
@@ -256,27 +289,34 @@ def start_collector(env, conf="collector_example.json"):
         SK = list(engine.stations.keys())
         tasks = []
         LSK = len(SK)
-        print("Stations on init", engine.stations,LSK)
-        for ids, sta in engine.stations.items():
-            print("Station", sta.get('code'), ids)
-            try:
-                queue_process.put(ids)
-            except Exception as exc:
-                print("Error al cargar en cola %s " % exc)
+
+        # add stations to assignator queue
+        # are added on method 'db_loop'
+        # for ids, sta in engine.stations.items():
+        #     code = sta.get('code')
+        #     try:
+        #         if sta.get("code") in cll_group or "ALL" in cll_group:
+        #             queue_process.put(ids)
+        #     except Exception as exc:
+        #         print("Error al cargar en cola %s " % exc)
 
         # Inicializa Assignator
 
         loop.run_in_executor(executor, assignator.new_process_task)
+        # run specific process for DATABASE SAVING
+
+        loop.run_in_executor(executor, engine.db_task) # 
 
         ####
         # Here we use the another (the rest) of the processors
         ####
+        #NIPT = 1 # for test
+        print(now(), "run_collector", f"Asignando tareas 'manage_tasks' -> {NIPT}")
         for i in range(NIPT):
             try:
                 # time.sleep(tsleep)
-                ipt = engine.set_ipt()
-                v = 1
-                # se inicializa lista de proc_task
+                ipt = engine.set_ipt(ipt=f"IPT_{i}")#engine.set_ipt()
+                # se inicializa lista de proc_task                
                 engine.proc_tasks[ipt] = []
                 try:
                     tipt = loop.run_in_executor(
@@ -284,7 +324,20 @@ def start_collector(env, conf="collector_example.json"):
                         functools.partial(
                             engine.manage_tasks,
                             ipt))
-                    gprint("ID ipt->%s" % tipt)
+                    message = f"Creation of process {ipt} for manage"+\
+                        f"tasks, Task {i}/{NIPT} -> {ipt}"
+                    queue_log.put(
+                        {"run_collector":[
+                            MessageLog(
+                                rdbnow(), 
+                                "run_collector", 
+                                message, 
+                                Logger.INFO, 
+                                exc).rdb
+                        ]
+                         }
+                    )                    
+                    gprint(f"ID ipt-> {tipt}")
                 except Exception as exc:
                     print("Error al activar nueva cpu")
                     print(exc)
@@ -302,17 +355,53 @@ def start_collector(env, conf="collector_example.json"):
 
         print("Done")
 
+from viztracer import VizTracer
+from viztracer import get_tracer
+
+        
 @click.command()
-@click.option("--name", default="collector", show_default=True, help="Nombre de la instancia collector a crear")
-@click.option("--env_vars/--no-env_vars", default=True, show_default=True,  type=bool, help="Para mostrar el nombre de las variables de ambiente" )
-@click.option("--env/--no-env", default=True, show_default=True,  type=bool, required=True,help="Si obtener los datos de ambiente o cargarlos de un json o data entregada")
-@click.option("--conf", default="JSON FILE",  show_default=True, help="Archivo json con los parámetros de database, debe contener las llaves {dbuser, dbpass, dbname, dbhost, dbport}")
-def run_collector(name, env_vars, env, conf):
-    print("Iniciando servicio COLLECTOR para %s" %name.upper())
+@click.option("--name", 
+              default="collector", 
+              show_default=True, 
+              help="Nombre de la instancia collector a crear")
+@click.option("--env_vars/--no-env_vars", 
+              default=True, 
+              show_default=True,  
+              type=bool, 
+              help="Para mostrar el nombre de las variables de ambiente" )
+@click.option("--env/--no-env", 
+              default=True, 
+              show_default=True,  
+              type=bool, 
+              required=True, 
+              help="Si obtener los datos de ambiente o cargarlos de un json o data entregada")
+@click.option("--conf", 
+              default="JSON FILE",  
+              show_default=True, 
+              help="Archivo json con los parámetros de database, debe contener las llaves {dbuser, dbpass, dbname, dbhost, dbport}")
+@click.option("--profile/--no-profile", 
+              default=False, 
+              show_default=True,  
+              type=bool, 
+              required=False, 
+              help="Generar tracing profile para debug")
+def run_collector(name, env_vars, env, conf, profile):
+    print(f"Iniciando servicio COLLECTOR para {name.upper()}")
     if env and env_vars:
         envvar = EnvData()
         envvar.show()
-    start_collector(env, conf)
 
+    if profile:
+        tracing_file = Path.home() / "profile.json"
+        tracer_entries=1000000
+        with VizTracer(
+                output_file=str(tracing_file),
+                tracer_entries=10_000_000,
+                log_async=True,
+                **get_tracer().init_kwargs) as tracer:        
+            start_collector(env, conf)
+    else:
+        start_collector(env, conf)
+        
 if __name__ == "__main__":
     run_collector()
